package ink.wgink.database.controller.api.mysql;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.database.pojo.dtos.mysql.MysqlDTO;
import ink.wgink.database.pojo.vos.mysql.MysqlVO;
import ink.wgink.database.service.mysql.IMysqlService;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: MysqlController
 * @Description: mysql连接
 * @Author: CodeFactory
 * @Date: 2021-10-26 23:05:32
 * @Version: 3.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "mysql连接接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/mysql")
public class MysqlController extends DefaultBaseController {

    @Autowired
    private IMysqlService mysqlService;

    @ApiOperation(value = "新增mysql连接", notes = "新增mysql连接接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult save(@RequestBody MysqlVO mysqlVO) {
        mysqlService.save(mysqlVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "删除mysql连接", notes = "删除mysql连接接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@PathVariable("ids") String ids) {
        mysqlService.remove(Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改mysql连接", notes = "修改mysql连接接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "mysqlId", value = "mysql连接ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("update/{mysqlId}")
    @CheckRequestBodyAnnotation
    public SuccessResult update(@PathVariable("mysqlId") String mysqlId, @RequestBody MysqlVO mysqlVO) {
        mysqlService.update(mysqlId, mysqlVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "mysql连接详情", notes = "mysql连接详情接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "mysqlId", value = "mysql连接ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{mysqlId}")
        public MysqlDTO get(@PathVariable("mysqlId") String mysqlId) {
        return mysqlService.get(mysqlId);
    }

    @ApiOperation(value = "mysql连接列表", notes = "mysql连接列表接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<MysqlDTO> list() {
        Map<String, Object> params = requestParams();
        return mysqlService.list(params);
    }

    @ApiOperation(value = "mysql连接分页列表", notes = "mysql连接分页列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
        @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
        @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<MysqlDTO>> listPage(ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return mysqlService.listPage(page);
    }

    @ApiOperation(value = "mysql连接统计", notes = "mysql连接统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("count")
    SuccessResultData<Integer> count() {
        Map<String, Object> params = requestParams();
        return new SuccessResultData<>(mysqlService.count(params));
    }

}