package ink.wgink.module.examine.service.question;

import ink.wgink.module.examine.pojo.bos.question.QuestionTypeBO;
import ink.wgink.module.examine.pojo.dtos.question.QuestionTypeDTO;
import ink.wgink.module.examine.pojo.pos.question.QuestionTypePO;
import ink.wgink.module.examine.pojo.vos.question.QuestionTypeBatchVO;
import ink.wgink.module.examine.pojo.vos.question.QuestionTypeVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.util.ReflectUtil;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IQuestionTypeService
 * @Description: 试题类型
 * @Author: CodeFactory
 * @Date: 2021-04-04 11:40:13
 * @Version: 3.0
 **/
public interface IQuestionTypeService {

    String UPDATE_TYPE_UPDATE = "update";
    String UPDATE_TYPE_REMOTE = "remove";

    /**
     * 新增试题类型
     *
     * @param questionTypeVO
     * @return
     */
    void save(QuestionTypeVO questionTypeVO) throws ReflectUtil.ReflectException;

    /**
     * 批量新增试题类型
     *
     * @param questionTypeBatchVO
     */
    void saveBatch(QuestionTypeBatchVO questionTypeBatchVO) throws ReflectUtil.ReflectException;

    /**
     * 新增试题类型
     *
     * @param token
     * @param questionTypeVO
     * @return
     */
    void save(String token, QuestionTypeVO questionTypeVO) throws ReflectUtil.ReflectException;

    /**
     * 新增试题类型
     *
     * @param questionTypeVO
     * @return questionTypeId
     */
    String saveReturnId(QuestionTypeVO questionTypeVO) throws ReflectUtil.ReflectException;

    /**
     * 新增试题类型
     *
     * @param token
     * @param questionTypeVO
     * @return questionTypeId
     */
    String saveReturnId(String token, QuestionTypeVO questionTypeVO) throws ReflectUtil.ReflectException;

    /**
     * 删除试题类型
     *
     * @param ids id列表
     * @return
     */
    void remove(List<String> ids);


    /**
     * 删除试题类型
     *
     * @param token
     * @param ids   id列表
     * @return
     */
    void remove(String token, List<String> ids);

    /**
     * 删除试题类型（物理删除）
     *
     * @param ids id列表
     */
    void delete(List<String> ids);

    /**
     * 修改试题类型
     *
     * @param questionTypeId
     * @param questionTypeVO
     * @return
     */
    void update(String questionTypeId, QuestionTypeVO questionTypeVO) throws ReflectUtil.ReflectException;

    /**
     * 修改试题类型
     *
     * @param token
     * @param questionTypeId
     * @param questionTypeVO
     * @return
     */
    void update(String token, String questionTypeId, QuestionTypeVO questionTypeVO) throws ReflectUtil.ReflectException;

    /**
     * 试题类型详情
     *
     * @param params 参数Map
     * @return
     */
    QuestionTypeDTO get(Map<String, Object> params);

    /**
     * 试题类型详情
     *
     * @param questionTypeId
     * @return
     */
    QuestionTypeDTO get(String questionTypeId);

    /**
     * 试题类型详情
     *
     * @param params 参数Map
     * @return
     */
    QuestionTypeBO getBO(Map<String, Object> params);

    /**
     * 试题类型详情
     *
     * @param questionTypeId
     * @return
     */
    QuestionTypeBO getBO(String questionTypeId);

    /**
     * 试题类型详情
     *
     * @param params 参数Map
     * @return
     */
    QuestionTypePO getPO(Map<String, Object> params);

    /**
     * 试题类型详情
     *
     * @param questionTypeId
     * @return
     */
    QuestionTypePO getPO(String questionTypeId);

    /**
     * 试题类型列表
     *
     * @param params
     * @return
     */
    List<QuestionTypeDTO> list(Map<String, Object> params);

    /**
     * 试题类型列表
     *
     * @param questionTypeIds
     * @return
     */
    List<QuestionTypeDTO> list(List<String> questionTypeIds);

    /**
     * 试题类型列表
     *
     * @param params
     * @return
     */
    List<QuestionTypeBO> listBO(Map<String, Object> params);

    /**
     * 试题类型列表
     *
     * @param params
     * @return
     */
    List<QuestionTypePO> listPO(Map<String, Object> params);

    /**
     * 试题类型分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<QuestionTypeDTO>> listPage(ListPage page);

    /**
     * 试题类型分页列表
     *
     * @param questionTypeParentId 上级ID列表
     * @param page                 分页
     * @return
     */
    SuccessResultList<List<QuestionTypeDTO>> listPage(String questionTypeParentId, ListPage page);

    /**
     * 试题类型统计
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 试题类型zTree列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

}