package ink.wgink.module.form.service.docno;

import ink.wgink.module.form.pojo.bos.docno.DocNoBO;
import ink.wgink.module.form.pojo.dtos.docno.DocNoDTO;
import ink.wgink.module.form.pojo.pos.docno.DocNoPO;
import ink.wgink.module.form.pojo.vos.docno.DocNoVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IDocNoService
 * @Description: 文号
 * @Author: CodeFactory
 * @Date: 2022-06-24 15:06:48
 * @Version: 3.0
 **/
public interface IDocNoService {

    /**
     * 新增文号
     *
     * @param docNoVO
     * @return
     */
    void save(DocNoVO docNoVO);

    /**
     * 新增文号
     *
     * @param token
     * @param docNoVO
     * @return
     */
    void save(String token, DocNoVO docNoVO);

    /**
     * 新增文号
     *
     * @param docNoVO
     * @return docNoId
     */
    String saveReturnId(DocNoVO docNoVO);

    /**
     * 新增文号
     *
     * @param token
     * @param docNoVO
     * @return docNoId
     */
    String saveReturnId(String token, DocNoVO docNoVO);

    /**
     * 删除文号
     *
     * @param ids id列表
     * @return
     */
    void remove(List<String> ids);


    /**
     * 删除文号
     *
     * @param token
     * @param ids   id列表
     * @return
     */
    void remove(String token, List<String> ids);

    /**
     * 删除文号（物理删除）
     *
     * @param ids id列表
     */
    void delete(List<String> ids);

    /**
     * 修改文号
     *
     * @param docNoId
     * @param docNoVO
     * @return
     */
    void update(String docNoId, DocNoVO docNoVO);

    /**
     * 修改文号
     *
     * @param token
     * @param docNoId
     * @param docNoVO
     * @return
     */
    void update(String token, String docNoId, DocNoVO docNoVO);

    /**
     * 文号详情
     *
     * @param params 参数Map
     * @return
     */
    DocNoDTO get(Map<String, Object> params);

    /**
     * 文号详情
     *
     * @param docNoId
     * @return
     */
    DocNoDTO get(String docNoId);

    /**
     * 文号详情
     *
     * @param params 参数Map
     * @return
     */
    DocNoPO getPO(Map<String, Object> params);

    /**
     * 文号详情
     *
     * @param docNoId
     * @return
     */
    DocNoPO getPO(String docNoId);

    /**
     * 文号列表
     *
     * @param params
     * @return
     */
    List<DocNoDTO> list(Map<String, Object> params);

    /**
     * 文号列表
     *
     * @param params
     * @return
     */
    List<DocNoPO> listPO(Map<String, Object> params);

    /**
     * 文号分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<DocNoDTO>> listPage(ListPage page);

    /**
     * 文号统计
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 获取最新文号
     *
     * @param docNoId
     * @param docNoUsedId 使用ID
     * @return
     */
    DocNoBO getLatestNewDocNoBO(String docNoId, String docNoUsedId);

    /**
     * 获取最新文号
     *
     * @param docNoPO
     * @param docNoUsedId 使用ID
     * @return
     */
    DocNoBO getLatestNewDocNoBO(DocNoPO docNoPO, String docNoUsedId);

    /**
     * 保存并返回新的文号
     *
     * @param docNoId
     * @param docNoUsedId 文号的使用ID，由调用方确定（每种文号确定唯一，动态表单用表名处理）
     * @return
     */
    String saveAndReturnLatestDocNo(String docNoId, String docNoUsedId);


}